@@ -488,6 +488,18 @@ class DESEnv:
             if isinstance(entity, Node) and entity.name == name:
                 return entity
 
+    @property
+    def network(self) -> "Network":
+        r"""Get the quantum network in this environment.
+
+        Returns:
+            Network: network in the discrete-event simulation environment
+        """
+        from qcompute_qnet.topology.network import Network
+        for entity in self.entities:
+            if isinstance(entity, Network):
+                return entity
+
     def set_log(self, path=None, level=None) -> None:
         r"""Log settings for the discrete-event simulation.
 
