@@ -128,8 +128,25 @@ class Channel(Entity, ABC):
         pass
 
 
-class ClassicalChannel(Channel, ABC):
+class ClassicalChannel(Channel):
     r"""Class for the simulation of classical channels.
+
+    Note:
+        ``ClassicalChannel`` is a logical classical channel in which no transmission loss will take place.
+         Thus, the lossy probability of a classical channel is set to be zero.
+
+    Examples:
+        We can create a classical channel between two nodes (e.g. alice and bob) by two approaches.
+        The first approach is to instantiate a ``ClassicalChannel`` with given ``name`` and ``distance``.
+        Then using the ``connect`` method to make a connection to alice and bob.
+
+        >>> c_a2b = ClassicalChannel(name="c_a2b", distance=1e3)
+        >>> c_a2b.connect(alice, bob)
+
+        The second approach is to assign the sender and receiver when instantiating.
+        This will make your code more compact.
+
+        >>> c_a2b = ClassicalChannel(name="c_a2b", distance=1e3, sender=alice, receiver=bob)
     """
 
     def __init__(self, name: str, distance: float, loss=0, env=None, sender=None, receiver=None, delay=None):
@@ -146,9 +163,36 @@ class ClassicalChannel(Channel, ABC):
         """
         super().__init__(name, distance, loss, env, sender, receiver, delay)
 
+    def transmit(self, msg: "ClassicalMessage", priority=None) -> None:
+        r"""Transmit classical message from the sender to the receiver.
 
-class QuantumChannel(Channel, ABC):
+        Args:
+            msg (ClassicalMessage): classical message to transmit
+            priority (int): priority of the transmission event
+        """
+        handler = EventHandler(self.receiver, "receive_classical_msg", [self.sender, msg])
+        self.scheduler.schedule_after(self.delay, handler, priority)
+
+
+class QuantumChannel(Channel):
     r"""Class for the simulation of quantum channels.
+
+    Note:
+        ``QuantumChannel`` is a logical quantum channel in which no transmission loss will take place.
+         Thus, the lossy probability of a quantum channel is set to be zero.
+
+    Examples:
+        We can create a quantum channel between two nodes (e.g. alice and bob) by two approaches.
+        The first approach is to instantiate a ``QuantumChannel`` with given ``name`` and ``distance``.
+        Then using the ``connect`` method to make a connection to alice and bob.
+
+        >>> q_a2b = QuantumChannel(name="q_a2b", distance=1e3)
+        >>> q_a2b.connect(alice, bob)
+
+        The second approach is to assign the sender and receiver when instantiating.
+        This will make your code more compact.
+
+        >>> q_a2b = QuantumChannel(name="q_a2b", distance=1e3, sender=alice, receiver=bob)
     """
 
     def __init__(self, name: str, distance: float, loss=0, env=None, sender=None, receiver=None, delay=None):
@@ -165,25 +209,22 @@ class QuantumChannel(Channel, ABC):
         """
         super().__init__(name, distance, loss, env, sender, receiver, delay)
 
+    def transmit(self, msg: "QuantumMessage", priority=None) -> None:
+        r"""Transmit quantum message from the sender to the receiver.
+
+        Args:
+            msg (QuantumMessage): quantum message to transmit
+            priority (int): priority of the transmission event
+        """
+        handler = EventHandler(self.receiver, "receive_quantum_msg", [self.sender, msg])
+        self.scheduler.schedule_after(self.delay, handler, priority)
+
 
 class ClassicalFiberChannel(ClassicalChannel):
     r"""Class for the simulation of classical fiber channels.
 
     Note:
-        The lossy probability of a clasical fiber channel is set to be zero for simplicity.
-
-    Examples:
-        We can create a classical fiber channel between two nodes (e.g. alice and bob) by two approaches.
-        The first approach is to instantiate a ``ClassicalFiberChannel`` with given ``name`` and ``distance``.
-        Then using the ``connect`` method to make a connection to alice and bob.
-
-        >>> c_a2b = ClassicalFiberChannel(name="c_a2b", distance=1e3)
-        >>> c_a2b.connect(alice, bob)
-
-        The second approach is to assign the sender and receiver when instantiating.
-        This will make your code more compact.
-
-        >>> c_a2b = ClassicalFiberChannel(name="c_a2b", distance=1e3, sender=alice, receiver=bob)
+        The lossy probability of a classical fiber channel is set to be zero for simplicity.
     """
 
     def __init__(self, name: str, distance: float, loss=0, env=None, sender=None, receiver=None, delay=None):
@@ -200,16 +241,6 @@ class ClassicalFiberChannel(ClassicalChannel):
         """
         super().__init__(name, distance, loss, env, sender, receiver, delay)
 
-    def transmit(self, msg: "ClassicalMessage", priority=None) -> None:
-        r"""Transmit classical message from the sender to the receiver.
-
-        Args:
-            msg (ClassicalMessage): classical message to transmit
-            priority (int): priority of the transmission event
-        """
-        handler = EventHandler(self.receiver, "receive_classical_msg", [self.sender, msg])
-        self.scheduler.schedule_after(self.delay, handler, priority)
-
 
 class QuantumFiberChannel(QuantumChannel):
     r"""Class for the simulation of quantum fiber channels.
@@ -219,19 +250,6 @@ class QuantumFiberChannel(QuantumChannel):
         The lossy probability is given by :math:`1 - 10^{- loss / 10}`, where loss (dB) is calculated by
         :math:`\alpha * L`, :math:`\alpha` is the attenuation (dB/km, default 0.2) and
         :math:`L` is the channel length (km).
-
-    Examples:
-        We can create a quantum fiber channel between two nodes (e.g. alice and bob) by two approaches.
-        The first approach is to instantiate a ``QuantumFiberChannel`` with given ``name`` and ``distance``.
-        Then using the ``connect`` method to make a connection to alice and bob.
-
-        >>> q_a2b = QuantumFiberChannel(name="q_a2b", distance=1e3)
-        >>> q_a2b.connect(alice, bob)
-
-        The second approach is to assign the sender and receiver when instantiating.
-        This will make your code more compact.
-
-        >>> q_a2b = QuantumFiberChannel(name="q_a2b", distance=1e3, sender=alice, receiver=bob)
     """
 
     def __init__(self, name: str, distance: float, loss=None, env=None, sender=None, receiver=None, delay=None):
@@ -260,8 +278,7 @@ class QuantumFiberChannel(QuantumChannel):
         """
         # Schedule an event for the receiver if the message is not lost
         if random.random_sample() > self.lossy_prob:
-            handler = EventHandler(self.receiver, "receive_quantum_msg", [self.sender, msg])
-            self.scheduler.schedule_after(self.delay, handler, priority)
+            super().transmit(msg, priority)
 
 
 class DuplexChannel(Entity, ABC):
