@@ -20,5 +20,6 @@ Module for template models.
 """
 
 __all__ = [
-    "qkd"
+    "qkd",
+    "qpu"
 ]
