@@ -86,6 +86,15 @@ class KeyGeneration(Protocol, ABC):
         if kwargs['peer'] == self.peer:
             self.start(**kwargs)
 
+    def receive_quantum_msg(self, msg: "QuantumMessage", **kwargs) -> None:
+        r"""Receive a quantum message from the node.
+
+        Args:
+            msg (Message): received quantum message
+            **kwargs (Any): keyword arguments for receiving the quantum message
+        """
+        self.node.polar_detector.receive(msg)
+
     def finish(self) -> None:
         r"""Finish the key generation protocol.
         """
@@ -171,35 +180,38 @@ class PrepareAndMeasure(KeyGeneration, ABC):
             else:
                 raise TypeError(f"Setting {attr} is not allowed in {self.name}")
 
-    def receive_message(self, msg: "ClassicalMessage") -> None:
+    def receive_classical_msg(self, msg: "ClassicalMessage", **kwargs) -> None:
         r"""Node receives a classical message.
 
         Args:
-            msg (ClassicalMessage): message used for the prepare-and-measure protocol
+            msg (ClassicalMessage): classical message used for the prepare-and-measure protocol
+            **kwargs (Any): keyword arguments for receiving the classical message
         """
         # Check if the message is for self and the protocol is working
         if msg.src == self.peer and self.status == KeyGeneration.Status.WORKING:
             # Different roles lead to different operations
             if self.role == PrepareAndMeasure.Role.TRANSMITTER:
-                self.transmitter_receive_message(msg)
+                self.transmitter_receive_classical_msg(msg)
             elif self.role == PrepareAndMeasure.Role.RECEIVER:
-                self.receiver_receive_message(msg)
+                self.receiver_receive_classical_msg(msg)
 
     @abstractmethod
-    def transmitter_receive_message(self, msg: "ClassicalMessage") -> None:
+    def transmitter_receive_classical_msg(self, msg: "ClassicalMessage", **kwargs) -> None:
         r"""Transmitter receives a classical message from the receiver.
 
         Args:
-            msg (ClassicalMessage): message for the prepare-and-measure protocol
+            msg (ClassicalMessage): classical message for the prepare-and-measure protocol
+            **kwargs (Any): keyword arguments for receiving the classical message
         """
         pass
 
     @abstractmethod
-    def receiver_receive_message(self, msg: "ClassicalMessage") -> None:
+    def receiver_receive_classical_msg(self, msg: "ClassicalMessage", **kwargs) -> None:
         r"""Receiver receives a classical message from the transmitter.
 
         Args:
-            msg (ClassicalMessage): message for the prepare-and-measure protocol
+            msg (ClassicalMessage): classical message for the prepare-and-measure protocol
+            **kwargs (Any): keyword arguments for receiving the classical message
         """
         pass
 
@@ -289,24 +301,24 @@ class BB84(PrepareAndMeasure):
         super().__init__(name, peer)
         self.basis_encoding = {0: Basis.Z(), 1: Basis.X()}
 
-    class BB84Message(ClassicalMessage):
+    class Message(ClassicalMessage):
         r"""Class for the classical control messages in BB84 protocol.
         """
 
         def __init__(self, src: "Node", dst: "Node", protocol: type, data: Dict):
-            r"""Constructor for BB84Message class.
+            r"""Constructor for Message class.
 
             Args:
-                src (Node): source of BB84Message
-                dst (Node): destination of BB84Message
-                protocol (type): protocol of BB84Message
+                src (Node): source of Message
+                dst (Node): destination of Message
+                protocol (type): protocol of Message
                 data (Dict): message content
             """
             super().__init__(src, dst, protocol, data)
 
         @unique
         class Type(Enum):
-            r"""Class for BB84Message types.
+            r"""Class for Message types.
             """
 
             START = "Start"
@@ -363,14 +375,14 @@ class BB84(PrepareAndMeasure):
         transmitter_emit_time = self.node.env.now + self.node.cchannel(self.peer).delay
         clock = transmitter_emit_time + self.node.qchannel(self.peer).delay  # clock time for photon reception
 
-        start_msg = BB84.BB84Message(src=self.node,
-                                     dst=self.peer,
-                                     protocol=BB84,
-                                     data={'type': BB84.BB84Message.Type.START,
-                                           'clock': clock,
-                                           'source_frequency': self.node.photon_source.frequency,
-                                           'light_duration': self.light_duration,
-                                           'emission_rest': self.emission_rest})
+        start_msg = BB84.Message(src=self.node,
+                                 dst=self.peer,
+                                 protocol=BB84,
+                                 data={'type': BB84.Message.Type.START,
+                                       'clock': clock,
+                                       'source_frequency': self.node.photon_source.frequency,
+                                       'light_duration': self.light_duration,
+                                       'emission_rest': self.emission_rest})
         self.node.send_classical_msg(dst=self.peer, msg=start_msg)
 
         # Transmitter begins photon emission as soon as the receiver receives the message
@@ -433,40 +445,41 @@ class BB84(PrepareAndMeasure):
             self.scheduler.schedule_at(clock + self.light_duration, handler)
 
             # Ask the transmitter to announce the bases for key sifting
-            request_bases_msg = BB84.BB84Message(src=self.node,
-                                                 dst=self.peer,
-                                                 protocol=BB84,
-                                                 data={'type': BB84.BB84Message.Type.BASES_REQUEST,
-                                                       'round': self.round})
+            request_bases_msg = BB84.Message(src=self.node,
+                                             dst=self.peer,
+                                             protocol=BB84,
+                                             data={'type': BB84.Message.Type.BASES_REQUEST,
+                                                   'round': self.round})
             self.node.send_classical_msg(dst=self.peer, msg=request_bases_msg)
 
         elif self.status == KeyGeneration.Status.DONE:
             self.node.polar_detector.detectors[0].turn_off()
             self.node.polar_detector.detectors[1].turn_off()
 
-    def transmitter_receive_message(self, msg: "BB84.BB84Message") -> None:
+    def transmitter_receive_classical_msg(self, msg: "BB84.Message", **kwargs) -> None:
         r"""Transmitter receives a BB84 message from the receiver.
 
         Args:
-            msg (BB84.BB84Message): message for BB84 protocol
+            msg (BB84.Message): message for BB84 protocol
+            **kwargs (Any): keyword arguments for receiving the classical message
         """
         photon_round = msg.data['round']
 
-        if msg.data['type'] == BB84.BB84Message.Type.BASES_REQUEST:
+        if msg.data['type'] == BB84.Message.Type.BASES_REQUEST:
             self.node.env.logger.debug(f"{self.node.name} received bases request from"
                                        f" {self.peer.name} (Round {photon_round})")
 
             # Pop the earliest bases and send to the receiver
             bases = self.tx_bases_list.pop(0)
-            bases_msg = BB84.BB84Message(src=self.node,
-                                         dst=self.peer,
-                                         protocol=BB84,
-                                         data={'type': BB84.BB84Message.Type.BASES,
-                                               'peer_bases': bases,
-                                               'round': photon_round})
+            bases_msg = BB84.Message(src=self.node,
+                                     dst=self.peer,
+                                     protocol=BB84,
+                                     data={'type': BB84.Message.Type.BASES,
+                                           'peer_bases': bases,
+                                           'round': photon_round})
             self.node.send_classical_msg(dst=self.peer, msg=bases_msg)
 
-        elif msg.data['type'] == BB84.BB84Message.Type.MATCHING:
+        elif msg.data['type'] == BB84.Message.Type.MATCHING:
             self.node.env.logger.debug(f"{self.node.name} received matching indices from {self.peer.name}"
                                        f" (Round {photon_round})")
 
@@ -491,13 +504,14 @@ class BB84(PrepareAndMeasure):
                     self.finish()  # transmitter finishes BB84 protocol
                     self.statistics_estimation()  # estimate sifted key rate and error rate
 
-    def receiver_receive_message(self, msg: "BB84.BB84Message") -> None:
+    def receiver_receive_classical_msg(self, msg: "BB84.Message", **kwargs) -> None:
         r"""Receiver receives a BB84 message from the transmitter.
 
         Args:
-            msg (BB84.BB84Message): message for BB84 protocol
+            msg (BB84.Message): message for BB84 protocol
+            **kwargs (Any): keyword arguments for receiving the classical message
         """
-        if msg.data['type'] == BB84.BB84Message.Type.START:
+        if msg.data['type'] == BB84.Message.Type.START:
             self.node.env.logger.debug(f"{self.node.name} received message 'START' from {self.peer.name}")
 
             clock = msg.data['clock']  # time for photon reception
@@ -520,7 +534,7 @@ class BB84(PrepareAndMeasure):
             handler = EventHandler(self, "receiver_stop_reception")
             self.scheduler.schedule_at(clock + self.light_duration, handler)
 
-        elif msg.data['type'] == BB84.BB84Message.Type.BASES:
+        elif msg.data['type'] == BB84.Message.Type.BASES:
             photon_round = msg.data['round']
             self.node.env.logger.debug(f"{self.node.name} received bases from {self.peer.name} "
                                        f"(Round: {photon_round})")
@@ -536,12 +550,12 @@ class BB84(PrepareAndMeasure):
                     self.rx_key_bits.append(bits[i])
 
             # Tell the transmitter the matching indices
-            matching_msg = BB84.BB84Message(src=self.node,
-                                            dst=self.peer,
-                                            protocol=BB84,
-                                            data={'type': BB84.BB84Message.Type.MATCHING,
-                                                  'matching_list': matching_list,
-                                                  'round': photon_round})
+            matching_msg = BB84.Message(src=self.node,
+                                        dst=self.peer,
+                                        protocol=BB84,
+                                        data={'type': BB84.Message.Type.MATCHING,
+                                              'matching_list': matching_list,
+                                              'round': photon_round})
             self.node.send_classical_msg(dst=self.peer, msg=matching_msg)
 
             # Check if the length of key bits is enough for a new key
@@ -602,24 +616,24 @@ class DecoyBB84(PrepareAndMeasure):
         DECOY = 1
         VACUUM = 2
 
-    class DecoyBB84Message(ClassicalMessage):
+    class Message(ClassicalMessage):
         r"""Class for the classical control messages in DecoyBB84 protocol.
         """
 
         def __init__(self, src: "Node", dst: "Node", protocol: type, data: Dict):
-            r"""Constructor for DecoyBB84Message class.
+            r"""Constructor for Message class.
 
             Args:
-                src (Node): source of DecoyBB84Message
-                dst (Node): destination of DecoyBB84Message
-                protocol (type): protocol of DecoyBB84Message
+                src (Node): source of Message
+                dst (Node): destination of Message
+                protocol (type): protocol of Message
                 data (Dict): message content
             """
             super().__init__(src, dst, protocol, data)
 
         @unique
         class Type(Enum):
-            r"""Class for DecoyBB84Message types.
+            r"""Class for Message types.
             """
 
             START = "Start"
@@ -685,14 +699,14 @@ class DecoyBB84(PrepareAndMeasure):
         transmitter_emit_time = self.node.env.now + self.node.cchannel(self.peer).delay
         clock = transmitter_emit_time + self.node.qchannel(self.peer).delay  # clock time for photon reception
 
-        start_msg = DecoyBB84.DecoyBB84Message(src=self.node,
-                                               dst=self.peer,
-                                               protocol=DecoyBB84,
-                                               data={'type': DecoyBB84.DecoyBB84Message.Type.START,
-                                                     'clock': clock,
-                                                     'source_frequency': self.node.photon_source.frequency,
-                                                     'light_duration': self.light_duration,
-                                                     'emission_rest': self.emission_rest})
+        start_msg = DecoyBB84.Message(src=self.node,
+                                      dst=self.peer,
+                                      protocol=DecoyBB84,
+                                      data={'type': DecoyBB84.Message.Type.START,
+                                            'clock': clock,
+                                            'source_frequency': self.node.photon_source.frequency,
+                                            'light_duration': self.light_duration,
+                                            'emission_rest': self.emission_rest})
         self.node.send_classical_msg(dst=self.peer, msg=start_msg)
 
         # Transmitter begins photon emission as soon as the receiver receives the message
@@ -740,11 +754,11 @@ class DecoyBB84(PrepareAndMeasure):
 
             handler = EventHandler(self.node.photon_source, "emit", [self.peer, states, mean_photon_num_list])
             self.scheduler.schedule_now(handler)
-            finish_emission_msg = DecoyBB84.DecoyBB84Message(src=self.node,
-                                                             dst=self.peer,
-                                                             protocol=DecoyBB84,
-                                                             data={'type': DecoyBB84.DecoyBB84Message.Type.FINISH_EMISSION,
-                                                                   'round': self.round})
+            finish_emission_msg = DecoyBB84.Message(src=self.node,
+                                                    dst=self.peer,
+                                                    protocol=DecoyBB84,
+                                                    data={'type': DecoyBB84.Message.Type.FINISH_EMISSION,
+                                                          'round': self.round})
             handler = EventHandler(self.node, "send_classical_msg", [self.peer, finish_emission_msg])
             self.scheduler.schedule_after(self.light_duration, handler)
 
@@ -772,12 +786,12 @@ class DecoyBB84(PrepareAndMeasure):
                     bases[i] = None
 
             # Send the measurement bases to the transmitter
-            bases_msg = DecoyBB84.DecoyBB84Message(src=self.node,
-                                                   dst=self.peer,
-                                                   protocol=DecoyBB84,
-                                                   data={'type': DecoyBB84.DecoyBB84Message.Type.BASES,
-                                                         'peer_bases': bases,
-                                                         'round': self.round})
+            bases_msg = DecoyBB84.Message(src=self.node,
+                                          dst=self.peer,
+                                          protocol=DecoyBB84,
+                                          data={'type': DecoyBB84.Message.Type.BASES,
+                                                'peer_bases': bases,
+                                                'round': self.round})
             self.node.send_classical_msg(dst=self.peer, msg=bases_msg)
 
             # Reset measurement bases for the next round of photon reception
@@ -793,16 +807,17 @@ class DecoyBB84(PrepareAndMeasure):
             self.node.polar_detector.detectors[0].turn_off()
             self.node.polar_detector.detectors[1].turn_off()
 
-    def transmitter_receive_message(self, msg: "DecoyBB84.DecoyBB84Message") -> None:
+    def transmitter_receive_classical_msg(self, msg: "DecoyBB84.Message", **kwargs) -> None:
         r"""Transmitter receives a DecoyBB84 message from the receiver.
 
         Args:
-            msg (DecoyBB84.DecoyBB84Message): message for DecoyBB84 protocol
+            msg (DecoyBB84.Message): message for DecoyBB84 protocol
+            **kwargs (Any): keyword arguments for receiving the classical message
         """
         photon_round = msg.data['round']
 
         # if self.node.env.now < self.end_time:
-        if msg.data['type'] == DecoyBB84.DecoyBB84Message.Type.BASES:
+        if msg.data['type'] == DecoyBB84.Message.Type.BASES:
             self.node.env.logger.debug(f"{self.node.name} received bases from {self.peer.name}"
                                        f" (Round {photon_round})")
 
@@ -844,18 +859,18 @@ class DecoyBB84(PrepareAndMeasure):
                     vacuum_bits.append(bit)
 
             # Send matching bases and testing bits with their positions to the receiver
-            matching_msg = DecoyBB84.DecoyBB84Message(src=self.node,
-                                                      dst=self.peer,
-                                                      protocol=DecoyBB84,
-                                                      data={'type': DecoyBB84.DecoyBB84Message.Type.MATCHING,
-                                                            'matching_list': matching_list,
-                                                            'decoy_pos': state_pos["decoy"],
-                                                            'vacuum_pos': state_pos["vacuum"],
-                                                            "testing_signal_pos": testing_signal_pos,
-                                                            "decoy_bits": decoy_bits,
-                                                            "vacuum_bits": vacuum_bits,
-                                                            "testing_signal_bits": testing_signal_bits,
-                                                            'round': photon_round})
+            matching_msg = DecoyBB84.Message(src=self.node,
+                                             dst=self.peer,
+                                             protocol=DecoyBB84,
+                                             data={'type': DecoyBB84.Message.Type.MATCHING,
+                                                   'matching_list': matching_list,
+                                                   'decoy_pos': state_pos["decoy"],
+                                                   'vacuum_pos': state_pos["vacuum"],
+                                                   "testing_signal_pos": testing_signal_pos,
+                                                   "decoy_bits": decoy_bits,
+                                                   "vacuum_bits": vacuum_bits,
+                                                   "testing_signal_bits": testing_signal_bits,
+                                                   'round': photon_round})
             self.node.send_classical_msg(dst=self.peer, msg=matching_msg)
 
             # Check if the length of key bits is enough for a new key
@@ -875,13 +890,14 @@ class DecoyBB84(PrepareAndMeasure):
                     # Calculate the overall sifted key rate in kilobit per second
                     self.key_rate_estimation()
 
-    def receiver_receive_message(self, msg: "DecoyBB84.DecoyBB84Message") -> None:
+    def receiver_receive_classical_msg(self, msg: "DecoyBB84.Message", **kwargs) -> None:
         r"""Receiver receives a DecoyBB84 message from the transmitter.
 
         Args:
-            msg (DecoyBB84.DecoyBB84Message): message for DecoyBB84 protocol
+            msg (DecoyBB84.Message): message for DecoyBB84 protocol
+            **kwargs (Any): keyword arguments for receiving the classical message
         """
-        if msg.data['type'] == DecoyBB84.DecoyBB84Message.Type.START:
+        if msg.data['type'] == DecoyBB84.Message.Type.START:
             self.node.env.logger.debug(f"{self.node.name} received message 'START' from {self.peer.name}")
 
             clock = msg.data['clock']  # time for photon reception
@@ -900,12 +916,12 @@ class DecoyBB84(PrepareAndMeasure):
             self.node.polar_detector.detectors[0].turn_on()
             self.node.polar_detector.detectors[1].turn_on()
 
-        elif msg.data['type'] == DecoyBB84.DecoyBB84Message.Type.FINISH_EMISSION:
+        elif msg.data['type'] == DecoyBB84.Message.Type.FINISH_EMISSION:
             # Stop receiving photons after a light duration
             handler = EventHandler(self, "receiver_stop_reception")
             self.scheduler.schedule_now(handler)
 
-        elif msg.data['type'] == DecoyBB84.DecoyBB84Message.Type.MATCHING:
+        elif msg.data['type'] == DecoyBB84.Message.Type.MATCHING:
             photon_round = msg.data['round']
             self.node.env.logger.debug(f"{self.node.name} received matching indices from {self.peer.name} "
                                        f"(Round: {photon_round})")
@@ -986,10 +1002,10 @@ class B92(PrepareAndMeasure):
         """
         super().__init__(name, peer)
 
-    def transmitter_receive_message(self, msg: "ClassicalMessage") -> None:
+    def transmitter_receive_classical_msg(self, msg: "ClassicalMessage", **kwargs) -> None:
         pass
 
-    def receiver_receive_message(self, msg: "ClassicalMessage") -> None:
+    def receiver_receive_classical_msg(self, msg: "ClassicalMessage", **kwargs) -> None:
         pass
 
 
@@ -1006,8 +1022,8 @@ class SixState(PrepareAndMeasure):
         """
         super().__init__(name, peer)
 
-    def transmitter_receive_message(self, msg: "ClassicalMessage") -> None:
+    def transmitter_receive_classical_msg(self, msg: "ClassicalMessage", **kwargs) -> None:
         pass
 
-    def receiver_receive_message(self, msg: "ClassicalMessage") -> None:
+    def receiver_receive_classical_msg(self, msg: "ClassicalMessage", **kwargs) -> None:
         pass
