@@ -42,6 +42,10 @@ __all__ = [
 
 class MBQC:
     r"""Class for the measurement-based quantum computation.
+
+    Attributes:
+        vertex (Vertex): vertices in MBQC algorithm
+        max_active (int): maximum number of active vertices
     """
 
     def __init__(self):
@@ -95,6 +99,9 @@ class MBQC:
     def set_graph(self, graph: List[List]) -> None:
         r"""Set the underlying graph of MBQC algorithm.
 
+        Args:
+            graph (List[List]): the underlying graph of MBQC algorithm
+
         Examples:
             The graph is given by a list as follows.
 
@@ -103,9 +110,6 @@ class MBQC:
             >>> E = [('1', '2'), ('2', '3'), ('3', '4'), ('4', '5')]  # a list of edges
             >>> G = [V, E]
             >>> mbqc.set_graph(G)
-
-        Args:
-            graph (List[List]): the underlying graph of MBQC algorithm
         """
         if not isinstance(graph, List):
             raise ArgumentTypeError(f"Input {graph} should be a list.")
@@ -153,6 +157,7 @@ class MBQC:
         cmd_num_wild = [cmd_map[cmd.name] for cmd in cmds]
         cmd_num_standard = cmd_num_wild[:]
         cmd_num_standard.sort(reverse=False)
+
         if not cmd_num_wild == cmd_num_standard:
             raise ArgumentTypeError(f"Input {pattern} is not a standard EMC pattern.")
 
@@ -213,6 +218,7 @@ class MBQC:
                         raise ArgumentTypeError(f"System label {label} should be an int value.")
                     if label < 0:
                         raise ArgumentTypeError(f"System label {label} should be an non-negative integer.")
+
                 systems = [label for label in self.__pattern.input_ if int(label[0]) in state.systems]
             else:
                 systems = state.systems
@@ -289,6 +295,7 @@ class MBQC:
         new_state = self.__bg_state
         new_state_len = new_state.length
         half_length = int(new_state_len / 2)
+
         if gate == 'X':
             gate_mat = Gate.X()
         elif gate == 'Z':
@@ -337,6 +344,7 @@ class MBQC:
         """
         self.__draw_process('measuring', which_qubit)
         self.__create_graph_state(which_qubit)
+
         if which_qubit not in self.vertex.active:
             raise ArgumentTypeError(f"Invalid qubit: ({which_qubit})! The qubit must be activated before measurement.")
 
@@ -428,6 +436,7 @@ class MBQC:
         if cmd.name not in ['M', 'X', 'Z']:
             raise ArgumentTypeError(f"Invalid command ({cmd}) with the name: ({cmd.name})!\n"
                                     f"Only 'M', 'X' and 'Z' are supported as the command name.")
+
         if cmd.name == 'M':  # execute measurement commands
             signal_s = self.sum_outcomes(cmd.domain_s)
             signal_t = self.sum_outcomes(cmd.domain_t)
@@ -462,14 +471,15 @@ class MBQC:
         """
         if not isinstance(cmd_s_lst, List):
             raise ArgumentTypeError(f"Input {cmd_s_lst} should be a list.")
+
         flip = {}
         for i in range(len(cmd_s_lst)):
             cmd_s = cmd_s_lst[i]
-
             # Execute signal shifting commands
             if cmd_s.which_qubit not in self.vertex.measured:
                 raise ArgumentTypeError(f"Invalid vertex index: ({cmd_s.which_qubit})!\n"
                                         f"This qubit is not measured.")
+
             power = self.sum_outcomes(cmd_s.domain)
             flip[cmd_s.which_qubit] = (self.__outcome[cmd_s.which_qubit] + power % 2) % 2
             print_progress((bar_start + i + 1) / bar_end, "Pattern Running Progress", self.__track)
@@ -481,7 +491,7 @@ class MBQC:
         r"""Kronecker product to all unmeasured qubits.
 
         Warning:
-            This is called when user runs MBQC from a measurement pattern.
+            This method is called when user runs MBQC from a measurement pattern.
         """
         self.__draw = False  # turn off the plot switch
         # As the create_graph_state function would change the measured qubits list, we need to record it
@@ -500,7 +510,7 @@ class MBQC:
         r"""Run the measurement pattern.
 
         Warning:
-            This is called after ``set_pattern``.
+            This method is called after ``set_pattern``.
         """
         if self.__pattern is None:
             raise ArgumentTypeError(f"Invalid pattern: ({self.__pattern})!\n"
@@ -579,7 +589,7 @@ class MBQC:
         """
         return self.__status
 
-    def __set_position(self, pos) -> None:
+    def __set_position(self, pos: Union[dict, bool]) -> None:
         r"""Set the position of graph plotting.
 
         Args:
@@ -693,6 +703,7 @@ class MBQC:
         """
         if not isinstance(track, bool):
             raise ArgumentTypeError(f"Input {track} should be a bool value.")
+
         self.__track = track
 
 
@@ -704,8 +715,12 @@ def run_circuit(circuit: "Circuit", shots=1024, input_state=None, optimize=True)
         and then runs the MBQC pattern to get equivalent sampling result.
         It can work well for large-scale quantum shallow circuits with the option ``optimize=True``.
 
+    Warnings:
+        We should check if the circuit has sequential registers first.
+        If not, we need to perform remapping before running the circuit.
+
     Args:
-        circuit (Circuit): quantum circuit
+        circuit (Circuit): quantum circuit to run
         shots (int, optional): number of sampling
         input_state (PureState, optional): input quantum state
         optimize (bool): whether to optimize the measurement order
@@ -713,15 +728,20 @@ def run_circuit(circuit: "Circuit", shots=1024, input_state=None, optimize=True)
     Returns:
         dict: classical results
     """
+    # Check if the circuit has sequential registers
+    if circuit.width != max(circuit.occupied_indices) + 1:
+        remap_circuit = circuit.copy()
+        remap_circuit.remap_indices()
+        circuit = remap_circuit
 
     # Add a layer of H gates to start from zero states
     if input_state is None:
         h_layer = [{"name": 'h', "which_qubit": [i], "params": None} for i in range(circuit.width)]
-        from qcompute_qnet.quantum.circuit import Circuit
-        new_circuit = Circuit(circuit.width)
+        new_circuit = circuit.copy()
         new_circuit._history = h_layer + circuit.gate_history
+        circuit = new_circuit
 
-    pattern = transpile_to_pattern(new_circuit, shift_signal=True, optimize=optimize, track=False)
+    pattern = transpile_to_pattern(circuit, shift_signal=True, optimize=optimize, track=False)
 
     samples = []
 
