@@ -57,10 +57,6 @@ class MCalculus:
         Args:
             circuit (Circuit): quantum circuit to set
         """
-
-        assert circuit.is_valid(), f"Invalid circuit: ({circuit})!\n" \
-                                   "There is at least one qubit not operated by any effective quantum gates."
-
         self.__circuit = circuit
         self.__circuit_width = circuit.width
 
@@ -110,7 +106,7 @@ class MCalculus:
                 if name == 'cz':  # input and output labels are the same
                     output_ = input_[:]
 
-                elif name == 'cnot':
+                elif name == 'cx':
                     output_ = [(control, int(counter.count(control))),
                                (target, int(counter.count(target) + 1))]
                     counter += [target]  # count the qubit index
@@ -121,7 +117,7 @@ class MCalculus:
                     counter += which_qubits  # count the qubit index
 
             # The gate after slicing has a form: [original_gate, input_, output_]
-            # E.g. = [[cnot, [0, 1], None], input_, output_]
+            # E.g. = [[cx, [0, 1], None], input_, output_]
             self.__circuit_slice.append([gate, input_, output_])
 
     @staticmethod
@@ -185,7 +181,6 @@ class MCalculus:
         original_gate, input_, output_ = gate
         name = original_gate["name"]
         which_qubit = original_gate["which_qubit"]
-        param = original_gate["params"]
 
         ancilla = []
 
@@ -254,7 +249,7 @@ class MCalculus:
             E12 = Pattern.CommandE([input_[0], ancilla[0]])
             E23 = Pattern.CommandE([ancilla[0], output_[0]])
             M1 = Pattern.CommandM(input_[0], 0, 'XY', [], [])
-            M2 = Pattern.CommandM(ancilla[0], - param, 'XY', [input_[0]], [])
+            M2 = Pattern.CommandM(ancilla[0], - original_gate["angle"], 'XY', [input_[0]], [])
             X3 = Pattern.CommandX(output_[0], [ancilla[0]])
             Z3 = Pattern.CommandZ(output_[0], [input_[0]])
             commands = [E12, E23, M1, M2, X3, Z3]
@@ -266,7 +261,7 @@ class MCalculus:
             E34 = Pattern.CommandE([ancilla[1], ancilla[2]])
             E45 = Pattern.CommandE([ancilla[2], output_[0]])
             M1 = Pattern.CommandM(input_[0], pi / 2, 'XY', [], [])
-            M2 = Pattern.CommandM(ancilla[0], - param, 'XY', [input_[0]], [])
+            M2 = Pattern.CommandM(ancilla[0], - original_gate["angle"], 'XY', [input_[0]], [])
             M3 = Pattern.CommandM(ancilla[1], - pi / 2, 'XY', [], [input_[0], ancilla[0]])
             M4 = Pattern.CommandM(ancilla[2], 0, 'XY', [], [ancilla[0]])
             X5 = Pattern.CommandX(output_[0], [ancilla[2]])
@@ -277,7 +272,7 @@ class MCalculus:
             ancilla = self.__set_ancilla(input_, output_, [1])
             E12 = Pattern.CommandE([input_[0], ancilla[0]])
             E23 = Pattern.CommandE([ancilla[0], output_[0]])
-            M1 = Pattern.CommandM(input_[0], - param, 'XY', [], [])
+            M1 = Pattern.CommandM(input_[0], - original_gate["angle"], 'XY', [], [])
             M2 = Pattern.CommandM(ancilla[0], 0, 'XY', [], [])
             X3 = Pattern.CommandX(output_[0], [ancilla[0]])
             Z3 = Pattern.CommandZ(output_[0], [input_[0]])
@@ -291,7 +286,7 @@ class MCalculus:
             E45 = Pattern.CommandE([ancilla[2], output_[0]])
             M1 = Pattern.CommandM(input_[0], 0, 'XY', [], [])
             M2 = Pattern.CommandM(ancilla[0], 0, 'XY', [], [])
-            M3 = Pattern.CommandM(ancilla[1], - param, 'XY', [ancilla[0]], [input_[0]])
+            M3 = Pattern.CommandM(ancilla[1], - original_gate["angle"], 'XY', [ancilla[0]], [input_[0]])
             M4 = Pattern.CommandM(ancilla[2], 0, 'XY', [], [ancilla[0]])
             X5 = Pattern.CommandX(output_[0], [ancilla[2]])
             Z5 = Pattern.CommandZ(output_[0], [ancilla[1]])
@@ -300,7 +295,7 @@ class MCalculus:
         elif name == 'u3':  # single-qubit unitary gate
             # Note: U = Rz(\phi) Rx(\theta) Rz(\lamda)
             ancilla = self.__set_ancilla(input_, output_, [3])
-            theta, phi, gamma = param
+            theta, phi, gamma = original_gate["angles"]
             E12 = Pattern.CommandE([input_[0], ancilla[0]])
             E23 = Pattern.CommandE([ancilla[0], ancilla[1]])
             E34 = Pattern.CommandE([ancilla[1], ancilla[2]])
@@ -316,7 +311,7 @@ class MCalculus:
         elif name == 'u':  # single-qubit unitary gate
             # Note: U = Rz(\phi) Ry(\theta) Rz(\lamda) usually used in MBQC
             ancilla = self.__set_ancilla(input_, output_, [3])
-            theta, phi, gamma = param
+            theta, phi, gamma = original_gate["angles"]
             E12 = Pattern.CommandE([input_[0], ancilla[0]])
             E23 = Pattern.CommandE([ancilla[0], ancilla[1]])
             E34 = Pattern.CommandE([ancilla[1], ancilla[2]])
@@ -329,7 +324,7 @@ class MCalculus:
             Z5 = Pattern.CommandZ(output_[0], [ancilla[1]])
             commands = [E12, E23, E34, E45, M1, M2, M3, M4, X5, Z5]
 
-        elif name == 'cnot':  # control NOT gate
+        elif name == 'cx':  # control NOT gate
             ancilla = self.__set_ancilla(input_, output_, [0, 1])
             E23 = Pattern.CommandE([input_[1], ancilla[0]])
             E13 = Pattern.CommandE([input_[0], ancilla[0]])
@@ -395,7 +390,7 @@ class MCalculus:
 
         elif name == 'm':  # single qubit measurement
             self.__measured_qubits.append(which_qubit[0])
-            commands = [Pattern.CommandM(input_[0], param[0], param[1], param[2], param[3])]
+            commands = [Pattern.CommandM(input_[0], *original_gate["basis"])]
 
         else:
             raise ArgumentTypeError(f"Invalid gate: ({gate})!\n"
@@ -904,9 +899,19 @@ def transpile_to_pattern(circuit: "Circuit", shift_signal=False, optimize=True,
         optimize (bool): whether to optimize the measurement orders
         track (bool): whether to track the transpiling progress
 
+    Warnings:
+        We should check if the circuit has sequential registers first.
+        If not, we need to perform remapping before transpiling the circuit.
+
     Returns:
         Pattern: an equivalent measurement pattern
     """
+    # Check if circuit has sequential registers
+    if circuit.width != max(circuit.occupied_indices) + 1:
+        new_circuit = circuit.copy()
+        new_circuit.remap_indices()
+        circuit = new_circuit
+
     mc = MCalculus()
     mc.track_progress(track)
     mc.set_circuit(circuit)
